@@ -1,10 +1,8 @@
 use super::common::*;
 use crate::circuit::mixer::MixerCircuit;
-use ark_crypto_primitives::SNARK;
+use ark_crypto_primitives::Error;
 use ark_ec::PairingEngine;
 use ark_ff::{BigInteger, PrimeField};
-use ark_groth16::{Groth16, Proof, ProvingKey, VerifyingKey};
-use ark_serialize::{CanonicalDeserialize, CanonicalSerialize};
 use ark_std::{
 	rand::{CryptoRng, Rng, RngCore},
 	rc::Rc,
@@ -17,7 +15,7 @@ use arkworks_gadgets::{
 };
 use arkworks_utils::{
 	poseidon::PoseidonParameters,
-	utils::common::{setup_params_x5_3, setup_params_x5_5, verify_groth16, Curve},
+	utils::common::{setup_params_x5_3, setup_params_x5_5, Curve},
 };
 
 pub type MixerConstraintDataInput<F> = MixerDataInput<F>;
@@ -62,45 +60,48 @@ pub type Circuit_MiMC220<F, const N: usize> = MixerCircuit<
 	N,
 >;
 
-pub fn setup_leaf<F: PrimeField, R: RngCore>(
-	params5: &PoseidonParameters<F>,
+pub fn setup_leaf_x5_5<F: PrimeField, R: RngCore>(
+	curve: Curve,
 	rng: &mut R,
-) -> (Vec<u8>, Vec<u8>, Vec<u8>, Vec<u8>) {
+) -> Result<(Vec<u8>, Vec<u8>, Vec<u8>, Vec<u8>), Error> {
+	let params5 = setup_params_x5_5::<F>(curve);
 	// Secret inputs for the leaf
 	let leaf_private = LeafPrivate::generate(rng);
 
-	let leaf_hash = Leaf_x5::create_leaf(&leaf_private, &params5).unwrap();
-	let nullifier_hash = Leaf_x5::create_nullifier(&leaf_private, &params5).unwrap();
+	let leaf_hash = Leaf_x5::create_leaf(&leaf_private, &params5)?;
+	let nullifier_hash = Leaf_x5::create_nullifier(&leaf_private, &params5)?;
 
 	let secret_bytes = leaf_private.secret().into_repr().to_bytes_le();
 	let nullifier_bytes = leaf_private.nullifier().into_repr().to_bytes_le();
 
 	let leaf_bytes = leaf_hash.into_repr().to_bytes_le();
 	let nullifier_hash_bytes = nullifier_hash.into_repr().to_bytes_le();
-	(
+	Ok((
 		secret_bytes,
 		nullifier_bytes,
 		leaf_bytes,
 		nullifier_hash_bytes,
-	)
+	))
 }
 
-pub fn setup_leaf_with_privates_raw<F: PrimeField>(
-	params5: &PoseidonParameters<F>,
+pub fn setup_leaf_with_privates_raw_x5_5<F: PrimeField>(
+	curve: Curve,
 	secret_bytes: Vec<u8>,
-	nullfier_bytes: Vec<u8>,
-) -> (Vec<u8>, Vec<u8>) {
+	nullifier_bytes: Vec<u8>,
+) -> Result<(Vec<u8>, Vec<u8>), Error> {
+	let params5 = setup_params_x5_5::<F>(curve);
+
 	let secret = F::from_le_bytes_mod_order(&secret_bytes);
-	let nullifier = F::from_le_bytes_mod_order(&nullfier_bytes);
+	let nullifier = F::from_le_bytes_mod_order(&nullifier_bytes);
 	// Secret inputs for the leaf
 	let leaf_private = LeafPrivate::new(secret, nullifier);
 
-	let leaf_hash = Leaf_x5::create_leaf(&leaf_private, &params5).unwrap();
-	let nullifier_hash = Leaf_x5::create_nullifier(&leaf_private, &params5).unwrap();
+	let leaf_hash = Leaf_x5::create_leaf(&leaf_private, &params5)?;
+	let nullifier_hash = Leaf_x5::create_nullifier(&leaf_private, &params5)?;
 
 	let leaf_bytes = leaf_hash.into_repr().to_bytes_le();
 	let nullifier_hash_bytes = nullifier_hash.into_repr().to_bytes_le();
-	(leaf_bytes, nullifier_hash_bytes)
+	Ok((leaf_bytes, nullifier_hash_bytes))
 }
 
 pub const LEN: usize = 30;
@@ -118,7 +119,7 @@ pub fn setup_proof_x5_5<E: PairingEngine, R: RngCore + CryptoRng>(
 	refund: u128,
 	pk: Vec<u8>,
 	rng: &mut R,
-) -> (Vec<u8>, Vec<u8>, Vec<u8>, Vec<u8>, Vec<Vec<u8>>) {
+) -> Result<(Vec<u8>, Vec<u8>, Vec<u8>, Vec<u8>, Vec<Vec<u8>>), Error> {
 	let params3 = setup_params_x5_3::<E::Fr>(curve);
 	let params5 = setup_params_x5_5::<E::Fr>(curve);
 	let prover = MixerProverSetupBn254_30::new(params3, params5);
@@ -133,17 +134,32 @@ pub fn setup_proof_x5_5<E: PairingEngine, R: RngCore + CryptoRng>(
 			relayer_raw,
 			fee,
 			refund,
-		);
+		)?;
 
-	let proof = MixerProverSetupBn254_30::<E::Fr>::prove_unchecked::<E, _>(circuit, &pk, rng);
+	let proof = prove_unchecked::<E, _, _>(circuit, &pk, rng)?;
 
-	(
+	Ok((
 		proof,
 		leaf_raw,
 		nullifier_hash_raw,
 		root_raw,
 		public_inputs_raw,
-	)
+	))
+}
+
+pub fn setup_keys_x5_5<E: PairingEngine, R: RngCore + CryptoRng>(
+	curve: Curve,
+	rng: &mut R,
+) -> Result<(Vec<u8>, Vec<u8>), Error> {
+	let params3 = setup_params_x5_3::<E::Fr>(curve);
+	let params5 = setup_params_x5_5::<E::Fr>(curve);
+	let prover = MixerProverSetupBn254_30::new(params3, params5);
+
+	let (circuit, ..) = prover.setup_random_circuit(rng)?;
+
+	let (pk, vk) = setup_keys_unchecked::<E, _, _>(circuit, rng)?;
+
+	Ok((pk, vk))
 }
 
 pub struct MixerProverSetup<F: PrimeField, const N: usize> {
@@ -196,69 +212,42 @@ impl<F: PrimeField, const N: usize> MixerProverSetup<F, N> {
 		)
 	}
 
-	pub fn setup_leaf<R: Rng>(&self, rng: &mut R) -> (LeafPrivate<F>, F, F) {
+	pub fn setup_leaf<R: Rng>(&self, rng: &mut R) -> Result<(LeafPrivate<F>, F, F), Error> {
 		// Secret inputs for the leaf
 		let leaf_private = LeafPrivate::generate(rng);
 
 		// Creating the leaf
-		let leaf_hash = Leaf_x5::create_leaf(&leaf_private, &self.params5).unwrap();
-		let nullifier_hash = Leaf_x5::create_nullifier(&leaf_private, &self.params5).unwrap();
-		(leaf_private, leaf_hash, nullifier_hash)
+		let leaf_hash = Leaf_x5::create_leaf(&leaf_private, &self.params5)?;
+		let nullifier_hash = Leaf_x5::create_nullifier(&leaf_private, &self.params5)?;
+		Ok((leaf_private, leaf_hash, nullifier_hash))
 	}
 
-	pub fn setup_leaf_with_privates(&self, secret: F, nullfier: F) -> (LeafPrivate<F>, F, F) {
+	pub fn setup_leaf_with_privates(
+		&self,
+		secret: F,
+		nullifier: F,
+	) -> Result<(LeafPrivate<F>, F, F), Error> {
 		// Secret inputs for the leaf
-		let leaf_private = LeafPrivate::new(secret, nullfier);
+		let leaf_private = LeafPrivate::new(secret, nullifier);
 
 		// Creating the leaf
-		let leaf_hash = Leaf_x5::create_leaf(&leaf_private, &self.params5).unwrap();
-		let nullifier_hash = Leaf_x5::create_nullifier(&leaf_private, &self.params5).unwrap();
-		(leaf_private, leaf_hash, nullifier_hash)
+		let leaf_hash = Leaf_x5::create_leaf(&leaf_private, &self.params5)?;
+		let nullifier_hash = Leaf_x5::create_nullifier(&leaf_private, &self.params5)?;
+		Ok((leaf_private, leaf_hash, nullifier_hash))
 	}
 
 	pub fn setup_leaf_with_privates_raw(
 		&self,
 		secret: Vec<u8>,
-		nullfier: Vec<u8>,
-	) -> (LeafPrivate<F>, F, F) {
+		nullifier: Vec<u8>,
+	) -> Result<(LeafPrivate<F>, F, F), Error> {
 		// Secret inputs for the leaf
 		let secret_f = F::from_le_bytes_mod_order(&secret);
-		let nullifier_f = F::from_le_bytes_mod_order(&nullfier);
+		let nullifier_f = F::from_le_bytes_mod_order(&nullifier);
 
 		self.setup_leaf_with_privates(secret_f, nullifier_f)
 	}
 
-	#[allow(clippy::too_many_arguments)]
-	pub fn setup_circuit<R: Rng>(
-		self,
-		leaves: &[F],
-		index: u64,
-		recipient: F,
-		relayer: F,
-		fee: F,
-		refund: F,
-		rng: &mut R,
-	) -> (Circuit_x5<F, N>, F, F, F, Vec<F>) {
-		let arbitrary_input = Self::setup_arbitrary_data(recipient, relayer, fee, refund);
-		let (leaf_private, leaf, nullifier_hash) = self.setup_leaf(rng);
-		let mut leaves_new = leaves.to_vec();
-		leaves_new.push(leaf);
-		let (tree, path) = self.setup_tree_and_create_path(&leaves_new, index);
-		let root = tree.root().inner();
-
-		let mc = Circuit_x5::new(
-			arbitrary_input,
-			leaf_private,
-			self.params5,
-			path,
-			root,
-			nullifier_hash,
-		);
-		let public_inputs =
-			Self::construct_public_inputs(nullifier_hash, root, recipient, relayer, fee, refund);
-		(mc, leaf, nullifier_hash, root, public_inputs)
-	}
-
 	#[allow(clippy::too_many_arguments)]
 	pub fn setup_circuit_with_privates(
 		self,
@@ -270,12 +259,11 @@ impl<F: PrimeField, const N: usize> MixerProverSetup<F, N> {
 		relayer: F,
 		fee: F,
 		refund: F,
-	) -> (Circuit_x5<F, N>, F, F, F, Vec<F>) {
+	) -> Result<(Circuit_x5<F, N>, F, F, F, Vec<F>), Error> {
 		let arbitrary_input = Self::setup_arbitrary_data(recipient, relayer, fee, refund);
-		let (leaf_private, leaf, nullifier_hash) = self.setup_leaf_with_privates(secret, nullifier);
-		let mut leaves_new = leaves.to_vec();
-		leaves_new.push(leaf);
-		let (tree, path) = self.setup_tree_and_create_path(&leaves_new, index);
+		let (leaf_private, leaf, nullifier_hash) =
+			self.setup_leaf_with_privates(secret, nullifier)?;
+		let (tree, path) = self.setup_tree_and_create_path(&leaves, index)?;
 		let root = tree.root().inner();
 
 		let mc = Circuit_x5::new(
@@ -288,7 +276,7 @@ impl<F: PrimeField, const N: usize> MixerProverSetup<F, N> {
 		);
 		let public_inputs =
 			Self::construct_public_inputs(nullifier_hash, root, recipient, relayer, fee, refund);
-		(mc, leaf, nullifier_hash, root, public_inputs)
+		Ok((mc, leaf, nullifier_hash, root, public_inputs))
 	}
 
 	#[allow(clippy::too_many_arguments)]
@@ -302,7 +290,7 @@ impl<F: PrimeField, const N: usize> MixerProverSetup<F, N> {
 		relayer: Vec<u8>,
 		fee: u128,
 		refund: u128,
-	) -> (Circuit_x5<F, N>, Vec<u8>, Vec<u8>, Vec<u8>, Vec<Vec<u8>>) {
+	) -> Result<(Circuit_x5<F, N>, Vec<u8>, Vec<u8>, Vec<u8>, Vec<Vec<u8>>), Error> {
 		let secret_f = F::from_le_bytes_mod_order(&secret);
 		let nullifier_f = F::from_le_bytes_mod_order(&nullifier);
 		let leaves_f: Vec<F> = leaves
@@ -323,7 +311,7 @@ impl<F: PrimeField, const N: usize> MixerProverSetup<F, N> {
 			relayer_f,
 			fee_f,
 			refund_f,
-		);
+		)?;
 
 		let leaf_raw = leaf.into_repr().to_bytes_le();
 		let nullifier_hash_raw = nullifier_hash.into_repr().to_bytes_le();
@@ -333,23 +321,33 @@ impl<F: PrimeField, const N: usize> MixerProverSetup<F, N> {
 			.map(|x| x.into_repr().to_bytes_le())
 			.collect();
 
-		(
+		Ok((
 			mc,
 			leaf_raw,
 			nullifier_hash_raw,
 			root_raw,
 			public_inputs_raw,
-		)
+		))
 	}
 
-	pub fn setup_random_circuit<R: Rng>(self, rng: &mut R) -> (Circuit_x5<F, N>, F, F, F, Vec<F>) {
-		let leaves = Vec::new();
-		let index = 0;
+	pub fn setup_random_circuit<R: Rng>(
+		self,
+		rng: &mut R,
+	) -> Result<(Circuit_x5<F, N>, F, F, F, Vec<F>), Error> {
 		let recipient = F::rand(rng);
 		let relayer = F::rand(rng);
 		let fee = F::rand(rng);
 		let refund = F::rand(rng);
-		self.setup_circuit(&leaves, index, recipient, relayer, fee, refund, rng)
+
+		let (leaf_privates, leaf_hash, ..) = self.setup_leaf(rng).unwrap();
+		let secret = leaf_privates.secret();
+		let nullifier = leaf_privates.nullifier();
+		let leaves = vec![leaf_hash];
+		let index = 0;
+
+		self.setup_circuit_with_privates(
+			secret, nullifier, &leaves, index, recipient, relayer, fee, refund,
+		)
 	}
 
 	pub fn create_circuit(
@@ -371,95 +369,21 @@ impl<F: PrimeField, const N: usize> MixerProverSetup<F, N> {
 		mc
 	}
 
-	pub fn setup_tree(&self, leaves: &[F]) -> Tree_x5<F> {
+	pub fn setup_tree(&self, leaves: &[F]) -> Result<Tree_x5<F>, Error> {
 		let inner_params = Rc::new(self.params3.clone());
-		let mt = Tree_x5::new_sequential(inner_params, Rc::new(()), leaves).unwrap();
-		mt
+		let mt = Tree_x5::new_sequential(inner_params, Rc::new(()), leaves)?;
+		Ok(mt)
 	}
 
 	pub fn setup_tree_and_create_path(
 		&self,
 		leaves: &[F],
 		index: u64,
-	) -> (Tree_x5<F>, Path<TreeConfig_x5<F>, N>) {
+	) -> Result<(Tree_x5<F>, Path<TreeConfig_x5<F>, N>), Error> {
 		// Making the merkle tree
-		let mt = self.setup_tree(leaves);
+		let mt = self.setup_tree(leaves)?;
 		// Getting the proof path
 		let path = mt.generate_membership_proof(index);
-		(mt, path)
-	}
-
-	pub fn setup_keys<E: PairingEngine, R: RngCore + CryptoRng>(
-		circuit: Circuit_x5<E::Fr, N>,
-		rng: &mut R,
-	) -> (Vec<u8>, Vec<u8>) {
-		let (pk, vk) = Groth16::<E>::circuit_specific_setup(circuit, rng).unwrap();
-
-		let mut pk_bytes = Vec::new();
-		let mut vk_bytes = Vec::new();
-		pk.serialize(&mut pk_bytes).unwrap();
-		vk.serialize(&mut vk_bytes).unwrap();
-		(pk_bytes, vk_bytes)
-	}
-
-	pub fn setup_keys_unchecked<E: PairingEngine, R: RngCore + CryptoRng>(
-		circuit: Circuit_x5<E::Fr, N>,
-		rng: &mut R,
-	) -> (Vec<u8>, Vec<u8>) {
-		let (pk, vk) = Groth16::<E>::circuit_specific_setup(circuit, rng).unwrap();
-
-		let mut pk_bytes = Vec::new();
-		let mut vk_bytes = Vec::new();
-		pk.serialize_unchecked(&mut pk_bytes).unwrap();
-		vk.serialize_unchecked(&mut vk_bytes).unwrap();
-		(pk_bytes, vk_bytes)
-	}
-
-	pub fn prove<E: PairingEngine, R: RngCore + CryptoRng>(
-		circuit: Circuit_x5<E::Fr, N>,
-		pk_bytes: &[u8],
-		rng: &mut R,
-	) -> Vec<u8> {
-		let pk = ProvingKey::<E>::deserialize(pk_bytes).unwrap();
-
-		let proof = Groth16::prove(&pk, circuit, rng).unwrap();
-		let mut proof_bytes = Vec::new();
-		proof.serialize(&mut proof_bytes).unwrap();
-		proof_bytes
-	}
-
-	pub fn prove_unchecked<E: PairingEngine, R: RngCore + CryptoRng>(
-		circuit: Circuit_x5<E::Fr, N>,
-		pk_unchecked_bytes: &[u8],
-		rng: &mut R,
-	) -> Vec<u8> {
-		let pk = ProvingKey::<E>::deserialize_unchecked(pk_unchecked_bytes).unwrap();
-
-		let proof = Groth16::prove(&pk, circuit, rng).unwrap();
-		let mut proof_bytes = Vec::new();
-		proof.serialize(&mut proof_bytes).unwrap();
-		proof_bytes
-	}
-
-	pub fn verify<E: PairingEngine>(
-		public_inputs: &[E::Fr],
-		vk_bytes: &[u8],
-		proof: &[u8],
-	) -> bool {
-		let vk = VerifyingKey::<E>::deserialize(vk_bytes).unwrap();
-		let proof = Proof::<E>::deserialize(proof).unwrap();
-		let ver_res = verify_groth16(&vk, &public_inputs, &proof);
-		ver_res
-	}
-
-	pub fn verify_unchecked<E: PairingEngine>(
-		public_inputs: &[E::Fr],
-		vk_unchecked_bytes: &[u8],
-		proof: &[u8],
-	) -> bool {
-		let vk = VerifyingKey::<E>::deserialize_unchecked(vk_unchecked_bytes).unwrap();
-		let proof = Proof::<E>::deserialize(proof).unwrap();
-		let ver_res = verify_groth16(&vk, &public_inputs, &proof);
-		ver_res
+		Ok((mt, path))
 	}
 }
